@@ -20,6 +20,7 @@ import sys
 AUTH_TOKEN = "foobar"
 AUTH_URL = "http://0.0.0.0"
 WS_URL = "ws://0.0.0.0"
+WSS_URL = "wss://0.0.0.0"
 
 
 class FakeApp(object):
@@ -62,5 +63,5 @@ class FakeClientWrapper(object):
     def __init__(self):
         self.ws = FakeWebSocket()
 
-    def messaging_websocket(self, queue_name="tripleo"):
+    def messaging_websocket(self):
         return self.ws
