@@ -22,6 +22,7 @@ import hashlib
 import logging
 import shutil
 
+import netaddr
 import os
 import os.path
 import simplejson
@@ -37,6 +38,7 @@ from heatclient.common import event_utils
 from heatclient.common import template_utils
 from heatclient.common import utils as heat_utils
 from heatclient.exc import HTTPNotFound
+from osc_lib import exceptions as oscexc
 from osc_lib.i18n import _
 from oslo_concurrency import processutils
 from six.moves import configparser
@@ -48,6 +50,8 @@ from six.moves.urllib import request
 from tripleoclient import constants
 from tripleoclient import exceptions
 
+LOG = logging.getLogger(__name__ + ".utils")
+
 
 def run_ansible_playbook(logger,
                          workdir,
@@ -135,6 +139,84 @@ def bracket_ipv6(address):
         return address
 
 
+def is_valid_ip(ip):
+    """Return True if the IP is either v4 or v6
+
+    Return False if invalid.
+    """
+    return netaddr.valid_ipv4(ip) or netaddr.valid_ipv6(ip)
+
+
+def is_loopback(host):
+    """Return True of the IP or the host is a loopback
+
+    Return False if not.
+    """
+    loopbacks = ['127', '::1']
+    for l in loopbacks:
+        if host.startswith(l):
+            return True
+    return False
+
+
+def get_host_ips(host, type=None):
+    """Lookup an host to return a list of IPs.
+
+    :param host: Host to lookup
+    :type  host: string
+
+    :param type: Type of socket (e.g. socket.AF_INET, socket.AF_INET6)
+    :type  type: string
+    """
+
+    ips = set()
+    if type:
+        types = (type,)
+    else:
+        types = (socket.AF_INET, socket.AF_INET6)
+    for t in types:
+        try:
+            res = socket.getaddrinfo(host, None, t, socket.SOCK_STREAM)
+        except socket.error:
+            continue
+        nips = set([x[4][0] for x in res])
+        ips.update(nips)
+    return list(ips)
+
+
+def get_single_ip(host, allow_loopback=False):
+    """Translate an hostname into a single IP address if it is a valid IP.
+
+    :param host: IP or hostname or FQDN to lookup
+    :type  host: string
+
+    :param allow_loopback: Whether or not a loopback IP can be returned.
+    Defaults is False.
+    :type  allow_loopback: boolean
+
+    Return the host unchanged if it is already an IPv4 or IPv6 address.
+    """
+
+    ip = host
+    if not is_valid_ip(host):
+        ips = get_host_ips(host)
+        if not ips:
+            raise exceptions.LookupError('No IP was found for the host: '
+                                         '%s' % host)
+        else:
+            ip = ips[0]
+        if len(ips) > 1:
+            raise exceptions.LookupError('More than one IP was found for the '
+                                         'host %s: %s' % (host, ips))
+        if not allow_loopback and is_loopback(ip):
+            raise exceptions.LookupError('IP address for host %s is a loopback'
+                                         ' IP: %s' % (host, ip))
+        if not is_valid_ip(ip):
+            raise exceptions.LookupError('IP address for host %s is not a '
+                                         'valid IP: %s' % (host, ip))
+    return ip
+
+
 def write_env_file(env_data, env_file, registry_overwrites):
     """Write the tht env file as yaml"""
 
@@ -491,6 +573,43 @@ def get_stack(orchestration_client, stack_name):
         pass
 
 
+def check_stack_network_matches_env_files(stack, environment):
+    """Check stack against proposed env files to ensure non-breaking change
+
+    Historically we have have had issues with folks forgetting the network
+    isolation templates in subsequent overcloud actions which have completely
+    broken the stack. We need to check that the networks continue to be
+    provided on updates and if they aren't, it's likely that the user has
+    failed to provide the network-isolation templates. This is a light check
+    to only ensure they are defined. A user can still change settings in these
+    networks that may break things but this will catch folks who forget
+    network-isolation in a subsequent update.
+    """
+    def _get_networks(registry):
+        nets = set()
+        for k, v in six.iteritems(registry):
+            if (k.startswith('OS::TripleO::Network::')
+                and not k.startswith('OS::TripleO::Network::Port')
+                    and v != 'OS::Heat::None'):
+                nets.add(k)
+        return nets
+
+    stack_registry = stack.environment().get('resource_registry', {})
+    env_registry = environment.get('resource_registry', {})
+
+    stack_nets = _get_networks(stack_registry)
+    env_nets = _get_networks(env_registry)
+
+    env_diff = set(stack_nets) - set(env_nets)
+    if env_diff:
+        raise exceptions.InvalidConfiguration('Missing networks from '
+                                              'environment configuration. '
+                                              'Ensure the following networks '
+                                              'are properly configured in '
+                                              'the provided environment files '
+                                              '[{}]'.format(env_diff))
+
+
 def remove_known_hosts(overcloud_ip):
     """For a given IP address remove SSH keys from the known_hosts file"""
 
@@ -1027,17 +1146,16 @@ def process_multiple_environments(created_env_files, tht_root,
 
 
 def run_update_ansible_action(log, clients, nodes, inventory, playbook,
-                              queue, all_playbooks, action, ssh_user,
-                              tags='', skip_tags=''):
+                              all_playbooks, action, ssh_user, tags='',
+                              skip_tags='', verbosity=1):
     playbooks = [playbook]
     if playbook == "all":
         playbooks = all_playbooks
     for book in playbooks:
         log.debug("Running ansible playbook %s " % book)
         action.update_ansible(clients, nodes=nodes, inventory_file=inventory,
-                              playbook=book, ansible_queue_name=queue,
-                              node_user=ssh_user, tags=tags,
-                              skip_tags=skip_tags)
+                              playbook=book, node_user=ssh_user, tags=tags,
+                              skip_tags=skip_tags, verbosity=verbosity)
 
 
 def prepend_environment(environment_files, templates_dir, environment):
@@ -1340,3 +1458,148 @@ def update_nodes_deploy_data(imageclient, nodes):
             if 'ramdisk_id' not in node and ramdisk in img_map:
                 node['ramdisk_id'] = img_map[ramdisk]
                 break
+
+
+def run_command(args, env=None, name=None, logger=None):
+    """Run the command defined by args and return its output
+
+    :param args: List of arguments for the command to be run.
+    :param env: Dict defining the environment variables. Pass None to use
+        the current environment.
+    :param name: User-friendly name for the command being run. A value of
+        None will cause args[0] to be used.
+    """
+    if logger is None:
+        logger = LOG
+    if name is None:
+        name = args[0]
+    try:
+        output = subprocess.check_output(args,
+                                         stderr=subprocess.STDOUT,
+                                         env=env)
+        if isinstance(output, bytes):
+            output = output.decode('utf-8')
+        return output
+    except subprocess.CalledProcessError as e:
+        message = '%s failed: %s' % (name, e.output)
+        logger.error(message)
+        raise RuntimeError(message)
+
+
+def set_hostname(hostname):
+    """Set system hostname to provided hostname
+
+    :param hostname: The hostname to set
+    """
+    args = ['sudo', 'hostnamectl', 'set-hostname', hostname]
+    return run_command(args, name='hostnamectl')
+
+
+def check_hostname(fix_etc_hosts=True, logger=None):
+    """Check system hostname configuration
+
+    Rabbit and Puppet require pretty specific hostname configuration. This
+    function ensures that the system hostname settings are valid before
+    continuing with the installation.
+
+    :param fix_etc_hosts: Boolean to to enable adding hostname to /etc/hosts
+        if not found.
+    """
+    if logger is None:
+        logger = LOG
+    logger.info('Checking for a FQDN hostname...')
+    args = ['hostnamectl', '--static']
+    detected_static_hostname = run_command(args, name='hostnamectl').rstrip()
+    logger.info('Static hostname detected as %s', detected_static_hostname)
+    args = ['hostnamectl', '--transient']
+    detected_transient_hostname = run_command(args,
+                                              name='hostnamectl').rstrip()
+    logger.info('Transient hostname detected as %s',
+                detected_transient_hostname)
+    if detected_static_hostname != detected_transient_hostname:
+        logger.error('Static hostname "%s" does not match transient hostname '
+                     '"%s".', detected_static_hostname,
+                     detected_transient_hostname)
+        logger.error('Use hostnamectl to set matching hostnames.')
+        raise RuntimeError('Static and transient hostnames do not match')
+    short_hostname = detected_static_hostname.split('.')[0]
+    if short_hostname == detected_static_hostname:
+        message = _('Configured hostname is not fully qualified.')
+        logger.error(message)
+        raise RuntimeError(message)
+    with open('/etc/hosts') as hosts_file:
+        for line in hosts_file:
+            # check if hostname is in /etc/hosts
+            if (not line.lstrip().startswith('#') and
+                    detected_static_hostname in line.split()):
+                break
+        else:
+            # hostname not found, add it to /etc/hosts
+            if not fix_etc_hosts:
+                return
+            sed_cmd = (r'sed -i "s/127.0.0.1\(\s*\)/127.0.0.1\\1%s %s /" '
+                       '/etc/hosts' %
+                       (detected_static_hostname, short_hostname))
+            args = ['sudo', '/bin/bash', '-c', sed_cmd]
+            run_command(args, name='hostname-to-etc-hosts')
+            logger.info('Added hostname %s to /etc/hosts',
+                        detected_static_hostname)
+
+
+def check_env_for_proxy(no_proxy_hosts=None):
+    """Check env proxy settings
+
+    :param no_proxy_hosts: array of hosts to check if in no_proxy env var
+    """
+    if no_proxy_hosts is None:
+        no_proxy_hosts = ['127.0.0.1']
+    http_proxy = os.environ.get('http_proxy', None)
+    https_proxy = os.environ.get('https_proxy', None)
+    if os.environ.get('no_proxy'):
+        no_proxy = os.environ.get('no_proxy').split(',')
+    else:
+        no_proxy = []
+    missing_hosts = []
+    if http_proxy or https_proxy:
+        missing_hosts = set(no_proxy_hosts) - set(no_proxy)
+    if missing_hosts:
+        message = _('http_proxy or https_proxy is set but the following local '
+                    'addresses "{}" may be missing from the no_proxy '
+                    'environment variable').format(','.join(missing_hosts))
+        raise RuntimeError(message)
+
+
+def check_file_for_enabled_service(env_file):
+    # This function checks environment file for the said service.
+    # If stack to be deployed/updated/upgraded has any deprecated service
+    # enabled, throw a warning about its deprecation and ask the user
+    # whether to proceed with deployment despite deprecation.
+    # For ODL as an example:
+    # If "OS::TripleO::Services::OpenDaylightApi" service is included
+    # in any of the parsed env_files, then check its value.
+    # OS::TripleO::Services::OpenDaylightApi NOT OS::Heat::None
+    # ODL is enabled.
+
+    if os.path.exists(env_file):
+        content = yaml.load(open(env_file))
+        deprecated_services_enabled = []
+        for service in constants.DEPRECATED_SERVICES.keys():
+            if ("resource_registry" in content and
+                    service in content["resource_registry"]):
+                if content["resource_registry"][service] != "OS::Heat::None":
+                    LOG.warn("service " + service + " is enabled in "
+                             + str(env_file) + ". " +
+                             constants.DEPRECATED_SERVICES[service])
+                    deprecated_services_enabled.append(service)
+
+        if deprecated_services_enabled:
+            confirm = prompt_user_for_confirmation(
+                message="Do you still wish to continue with deployment [y/N]",
+                logger=LOG)
+            if not confirm:
+                raise oscexc.CommandError("Action not confirmed, exiting.")
+
+
+def check_deprecated_service_is_enabled(environment_files):
+    for env_file in environment_files:
+        check_file_for_enabled_service(env_file)
