@@ -54,9 +54,9 @@ class ExternalUpdateRun(command.Command):
         parser.add_argument("--ssh-user",
                             dest="ssh_user",
                             action="store",
-                            default="heat-admin",
-                            help=_("The ssh user name for connecting to "
-                                   "the overcloud nodes.")
+                            default="tripleo-admin",
+                            help=_("DEPRECATED: Only tripleo-admin should be "
+                                   "used as ssh user.")
                             )
         parser.add_argument('--tags',
                             dest='tags',
@@ -85,6 +85,7 @@ class ExternalUpdateRun(command.Command):
     def take_action(self, parsed_args):
         self.log.debug("take_action(%s)" % parsed_args)
         clients = self.app.client_manager
+        verbosity = self.app_args.verbose_level
         stack = parsed_args.stack
 
         # Run ansible:
@@ -94,9 +95,9 @@ class ExternalUpdateRun(command.Command):
         playbook = 'all'
         oooutils.run_update_ansible_action(
             self.log, clients, limit_hosts, inventory, playbook,
-            constants.EXTERNAL_UPDATE_QUEUE,
             constants.EXTERNAL_UPDATE_PLAYBOOKS,
             package_update, parsed_args.ssh_user,
-            tags=parsed_args.tags, skip_tags=parsed_args.skip_tags)
+            tags=parsed_args.tags, skip_tags=parsed_args.skip_tags,
+            verbosity=verbosity)
 
         self.log.info("Completed Overcloud External Update Run.")
